# -*- coding: utf-8 -*-
import difflib
from App.class_init import InitializeClass

from Products.CMFDiffTool.FieldDiff import FieldDiff
from Products.CMFDiffTool.utils import safe_unicode, safe_utf8


class TextDiff(FieldDiff):
    """Text difference"""

    meta_type = "Lines Diff"
    inlinediff_fmt = """
<div class="InlineDiff">
    <p><del>%s</del></p><del>%s</del>
    <p><ins>%s</ins></p><ins>%s</ins>
</div>
"""

    def _parseField(self, value, filename=None):
        """Parse a field value in preparation for diffing"""
        if filename is None:
            # Split the text into a list for diffs
            return value.split('\n')
        else:
            return [self.filenameTitle(filename)] + value.split('\n')

    def unified_diff(self):
        """Return a unified diff"""
        a = [safe_utf8(i) for i in
             self._parseField(self.oldValue, filename=self.oldFilename)]
        b = [safe_utf8(i) for i in
             self._parseField(self.newValue, filename=self.newFilename)]
        return '\n'.join(difflib.unified_diff(a, b, self.id1, self.id2))

    def html_diff(self, context=True, wrapcolumn=40):
        """Return an HTML table showing differences"""
        # difflib is not Unicode-aware, so we need to force everything to
        # utf-8 manually
        a = [safe_unicode(i) for i in
             self._parseField(self.oldValue, filename=self.oldFilename)]
        b = [safe_unicode(i) for i in
             self._parseField(self.newValue, filename=self.newFilename)]
        vis_diff = difflib.HtmlDiff(wrapcolumn=wrapcolumn)
        diff = safe_utf8(vis_diff.make_table(a, b,
                                             safe_unicode(self.id1),
                                             safe_unicode(self.id2),
                                             context=context))
        return diff

    def inline_diff(self):
        """Simple inline diff that just assumes that the text has been
        completely replaced."""
        old_attr = self._parseField(self.oldValue,
                                    filename=self.oldFilename)
        new_attr = self._parseField(self.newValue,
                                    filename=self.newFilename)
        old_fname = old_attr.pop(0)
        new_fname = new_attr.pop(0)
        a = '\n'.join(old_attr)
        b = '\n'.join(new_attr)
        if self.oldValue != self.newValue or \
           self.oldFilename != self.newFilename:
            return self.inlinediff_fmt % (old_fname, a, new_fname, b)


InitializeClass(TextDiff)
